
// This code uses the SevSeg library, which can be donwnloaded from:
// https://github.com/sparkfun/SevSeg
#include "SevSeg.h" //Library to control generic seven segment displays

SevSeg myDisplay;           //Create an instance of the object

long lastMillis = 0;        //Keeps track of the last time we recorded (used to measure how long our interval is)
int countSec = 0;           //Contains the tenth of seconds (right most two digits of the 7-segment) 
int countMin = 0;           //Contains the seconds (left most two digits of the 7-segment)

bool buttonState = true;    //The state of the button plate (drink plate) defaulted to true
bool previousState = true;  //The previous state of the button plate (drink plate) so we can track when a state change occurs (when a glass is placed or removed from the plate)
int stateCount = 0;         //Contains the number of state changes that occur (steps us through our state machine in the loop code)

// Struct for 4-digit, 7-segment display
// Stores display value (digits),  decimal status (decimals) for each digit, and cursor for overall display
struct display
{
  char digits[4];
  unsigned char decimals;
  unsigned char cursor;
}
display;

void setup()
{
  //Initialize display stuff (common cathode, digits, brightness, etc)
  setupDisplay(); 

  //Preload the display buffer with a default
  display.digits[0] = 0;
  display.digits[1] = 0;
  display.digits[2] = 0;
  display.digits[3] = 0;
  display.decimals = '0b00010000';

  //start the serial connection for debugging purposes
  Serial.begin(9600);

  //Set the pin we've connected our button plate to as an input
  pinMode(A7, INPUT); 
}

// The display is constantly PWM'd in the loop()
void loop()
{
  //If we detect that a drink has been placed onto the tray, set the state to "true". Or if not, set it to "false"
  if (analogRead(A7) >= 500) 
  {
    buttonState = true;
  }
  else
  {
    buttonState = false;
  }

  //If the state of the button plate has changed (a drink has been added or removed), increment the state counter by 1.
  if (buttonState != previousState) 
  {
    //If we have reached the end of our state machines state count, reset the counter
    if (stateCount >= 4) 
    {
      stateCount = 0;
    }
    else
    {
      stateCount += 1;
      previousState = buttonState;
    }
    Serial.print("State Count: ");
    Serial.println(stateCount);
  }


  //This is our state machine that determines what we will do when the state counter is changed. Each "case" has a different function and when the state counter reaches the end of the
  //state machine, the counter is reset back to zero so that we can start the process all over again.
  switch (stateCount)
  {
    case 0: //no glass, waiting for full glass to be placed. Reset the display back to zero's and display the data.
      display.digits[0] = 0;
      display.digits[1] = 0;
      display.digits[2] = 0;
      display.digits[3] = 0;
      displayData();
      break;
    case 1: //glass placed, waiting for release. do not start the timer but continue displaying the zero's.
      displayData();
      break;
    case 2: //glass removed, timer started. Every time the loop "loops" the timer is incremented by a 10th of a second.
      chuggTime("go");
      displayData();
      break;
    case 3: //glass finished, timer stopped. Stop incrementing the timer so that you can view your "score".
      displayData();
      break;
    case 4: //empty glass removed. reset the timer variables so that the next time we loop through the statemachine, the timer will start at zero.
      displayData();
      countSec = 0;
      countMin = 0;
      break;
    default: //something went wrong. Just display the timer and hope nobody notices...
      displayData();
      break;
  }

  //Delay 1 millisecond (this is roughly the display's refresh rate. If this value is too big, the display will have significant flickering
  delay(1); 
}

//Display whatever data we have entered in the display digits and decimals
void displayData()
{
  myDisplay.DisplayString(display.digits, display.decimals); //(numberToDisplay, decimal point location)
  delay(10);
}

//Time the chugging so that the display can be updated every 100 milliseconds
void chuggTime(String state)
{
  //if a state of "go" has been passed to the chuggtime method, start the counting process
  if (state == "go")
  {
    //if 100 milliseconds have passed since the last time we checked
    if (millis() - lastMillis >= 100)
    {
      //Set the new "last time checked" as the time it is now
      lastMillis = millis();

      //increment the counter by a 10th of a second
      countSec += 10;

      //if 10 counts of a 10th of a second have passed, increment countMin by 1 to show that it's been and additional second. 
      if (countSec >= 100)
      {
        countSec = 0;
        countMin += 1;
      }

      //if countSec is less than 9, the 2 right most digits of the display will show 01, 02, 03, 04, ...etc
      if (countSec <= 9)
      {
        display.digits[3] = countSec;
        display.digits[2] = 0;
      }
      else //If count Sec is larger than 9 the display will show 10, 11, 12, ... 21, 22, 23, 24, ... etc
      {
        display.digits[3] = (countSec % 10);
        display.digits[2] = (countSec / 10);
      }

      if (countMin <= 9) //same as above
      {
        display.digits[1] = countMin;
        display.digits[0] = 0;
      }
      else
      {
        display.digits[1] = (countMin % 10);
        display.digits[0] = (countMin / 10);
      }
    }
  }
  else
  {
    //do nothing
  }
}

//Sets up the hardware pins to control the 7 segments and display type
//This code was imported from sparkfuns display firmware:
//Version: 3.0.1
//By: Jim Lindblom (SparkFun Electronics)
void setupDisplay()
{
  myDisplay.SetBrightness(100); //Set the display to 100% bright

  // Set the initial state of displays and decimals 'x' =  off
  display.digits[0] = 'x';
  display.digits[1] = 'x';
  display.digits[2] = 'x';
  display.digits[3] = 'x';
  display.decimals = 0x00;  // Turn all decimals off
  display.cursor = 0;  // Set cursor to first (left-most) digit

  int digit1 = 16; // DIG1 = A2/16 (PC2)
  int digit2 = 17; // DIG2 = A3/17 (PC3)
  int digit3 = 3;  // DIG3 = D3 (PD3)
  int digit4 = 4;  // DIG4 = D4 (PD4)

  //Declare what pins are connected to the segments
  int segA = 8;  // A = D8 (PB0)
  int segB = 14; // B = A0 (PC0)
  int segC = 6;  // C = D6 (PD6), shares a pin with colon cathode
  int segD = A1; // D = A1 (PC1)
  int segE = 23; // E = PB7 (not a standard Arduino pin: Must add PB7 as digital pin 23 to pins_arduino.h)
  int segF = 7;  // F = D7 (PD6), shares a pin with apostrophe cathode
  int segG = 5;  // G = D5 (PD5)
  int segDP = 22; //DP = PB6 (not a standard Arduino pin: Must add PB6 as digital pin 22 to pins_arduino.h)

  int digitColon = 2; // COL-A = D2 (PD2) (anode of colon)
  int segmentColon = 6; // COL-C = D6 (PD6) (cathode of colon), shares a pin with C
  int digitApostrophe = 9; // APOS-A = D9 (PB1) (anode of apostrophe)
  int segmentApostrophe = 7; // APOS-C = D7 (PD7) (cathode of apostrophe), shares a pin with F

  int numberOfDigits = 4; //Do you have a 2 or 4 digit display?

  int displayType = COMMON_ANODE; //SparkFun 10mm height displays are common anode

  //Initialize the SevSeg library with all the pins needed for this type of display
  myDisplay.Begin(displayType, numberOfDigits,
                  digit1, digit2, digit3, digit4,
                  digitColon, digitApostrophe,
                  segA, segB, segC, segD, segE, segF, segG,
                  segDP,
                  segmentColon, segmentApostrophe);
}
